/**
 * Copyright (c) 2014 - 2018, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 * Peripheral: SAADC
 * Compatibility: nRF52832 rev 2/nRF52840 rev 1, SDK 15.2.0
 * Softdevice used: S132 v6.1.0/S140 v6.1.0
 *
 * This SAADC example samples on 4 different input pins, and enables scan mode to do that. It is otherwise an
 * offsprint from the standard ble_app_uart example available in nRF5 SDK 15.2.0
 * Works together with softdevice S132 v6.1.0 on nRF52832 and S140 v6.1.0 on nRF52840
 * Transmits SAADC output to hardware UART and over BLE via Nordic UART Servive (NUS).
 * Info on NUS -> http://infocenter.nordicsemi.com/topic/com.nordic.infocenter.sdk5.v13.0.0/ble_sdk_app_nus_eval.html?cp=4_0_0_4_1_2_17
 * Info on hardware UART settings -> http://infocenter.nordicsemi.com/topic/com.nordic.infocenter.sdk5.v13.0.0/uart_example.html?cp=4_0_0_4_4_41
 */


#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_qwr.h"
#include "app_timer.h"
#include "ble_nus.h"
#include "app_uart.h"
#include "app_util_platform.h"
#include "bsp_btn_ble.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_drv_saadc.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_timer.h"
#include "peer_manager.h"
#include "peer_manager_handler.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"

#include "app_pwm.h"

#include "nrf_calendar.h"

#if defined (UART_PRESENT)
#include "nrf_uart.h"
#endif
#if defined (UARTE_PRESENT)
#include "nrf_uarte.h"
#endif

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#define APP_BLE_CONN_CFG_TAG            1                                           /**< A tag identifying the SoftDevice BLE configuration. */

#define DEVICE_NAME                     "IEDC_BRD2"                               /**< Name of device. Will be included in the advertising data. */
#define NUS_SERVICE_UUID_TYPE           BLE_UUID_TYPE_VENDOR_BEGIN                  /**< UUID type for the Nordic UART Service (vendor specific). */

#define APP_BLE_OBSERVER_PRIO           3                                           /**< Application's BLE observer priority. You shouldn't need to modify this value. */

#define APP_ADV_INTERVAL                64                                          /**< The advertising interval (in units of 0.625 ms. This value corresponds to 40 ms). */

#define APP_ADV_DURATION                18000                                       /**< The advertising duration (180 seconds) in units of 10 milliseconds. */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(20, UNIT_1_25_MS)             /**< Minimum acceptable connection interval (20 ms), Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(75, UNIT_1_25_MS)             /**< Maximum acceptable connection interval (75 ms), Connection interval uses 1.25 ms units. */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)             /**< Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units. */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000)                       /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000)                      /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                           /**< Number of attempts before giving up the connection parameter negotiation. */

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define UART_TX_BUF_SIZE                256                                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                256                                         /**< UART RX buffer size. */

#define SAADC_SAMPLES_IN_BUFFER         4
#define SAADC_SAMPLE_RATE		250                                         /**< SAADC sample rate in ms. */               


#define BRD_LED1 8
#define BRD_LED2 11
#define BRD_LED3 12
#define BRD_LED4 14

#define MOT_CTL 7

#define BAT_STATE 6
#define KEY_PIN 30

#define SEC_PARAM_BOND                  1                                           /**< Perform bonding. */
#define SEC_PARAM_MITM                  1                                           /**< Man In The Middle protection required (applicable when display module is detected). */
#define SEC_PARAM_LESC                  1                                           /**< LE Secure Connections enabled. */
#define SEC_PARAM_KEYPRESS              0                                           /**< Keypress notifications not enabled. */
#define SEC_PARAM_IO_CAPABILITIES       BLE_GAP_IO_CAPS_DISPLAY_ONLY                /**< Display I/O capabilities. */
#define SEC_PARAM_OOB                   0                                           /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE          7                                           /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE          16                                          /**< Maximum encryption key size. */

#define PASSKEY_TXT                     "Passkey:"                                  /**< Message to be displayed together with the pass-key. */
#define PASSKEY_TXT_LENGTH              8                                           /**< Length of message to be displayed together with the pass-key. */
#define PASSKEY_LENGTH                  6                                           /**< Length of pass-key received by the stack for display. */

static pm_peer_id_t m_peer_to_be_deleted = PM_PEER_ID_INVALID;
static uint16_t     m_conn_handle        = BLE_CONN_HANDLE_INVALID;                 /**< Handle of the current connection. */


static void pm_evt_handler(pm_evt_t const * p_evt)
{
    ret_code_t err_code;

    pm_handler_on_pm_evt(p_evt);
    pm_handler_disconnect_on_sec_failure(p_evt);
    pm_handler_flash_clean(p_evt);

    switch (p_evt->evt_id)
    {
        case PM_EVT_CONN_SEC_SUCCEEDED:
        {
            pm_conn_sec_status_t conn_sec_status;

            // Check if the link is authenticated (meaning at least MITM).
            err_code = pm_conn_sec_status_get(p_evt->conn_handle, &conn_sec_status);
            APP_ERROR_CHECK(err_code);

            if (conn_sec_status.mitm_protected)
            {
                NRF_LOG_INFO("Link secured. Role: %d. conn_handle: %d, Procedure: %d",
                             ble_conn_state_role(p_evt->conn_handle),
                             p_evt->conn_handle,
                             p_evt->params.conn_sec_succeeded.procedure);
            }
            else
            {
                // The peer did not use MITM, disconnect.
                NRF_LOG_INFO("Collector did not use MITM, disconnecting");
                err_code = pm_peer_id_get(m_conn_handle, &m_peer_to_be_deleted);
                APP_ERROR_CHECK(err_code);
                err_code = sd_ble_gap_disconnect(m_conn_handle,
                                                 BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
                APP_ERROR_CHECK(err_code);
            }
        } break;

        case PM_EVT_CONN_SEC_FAILED:
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            break;

        case PM_EVT_PEERS_DELETE_SUCCEEDED:
            advertising_start(false);
            break;

        default:
            break;
    }
}


typedef struct
{
    int16_t z;
    int16_t y;
    int16_t x;
}accel_values_t;

BLE_NUS_DEF(m_nus, NRF_SDH_BLE_TOTAL_LINK_COUNT);                                   /**< BLE NUS service instance. */
NRF_BLE_GATT_DEF(m_gatt);                                                           /**< GATT module instance. */
NRF_BLE_QWR_DEF(m_qwr);                                                             /*< Context for the Queued Write module.*/
BLE_ADVERTISING_DEF(m_advertising);                                                 /**< Advertising module instance. */

                /**< Handle of the current connection. */
static uint16_t   m_ble_nus_max_data_len = BLE_GATT_ATT_MTU_DEFAULT - 3;            /**< Maximum length of data (in bytes) that can be transmitted to the peer by the Nordic UART service module. */
static ble_uuid_t m_adv_uuids[]          =                                          /**< Universally unique service identifier. */
{
    {BLE_UUID_NUS_SERVICE, NUS_SERVICE_UUID_TYPE}
};

static dc = 50;
static kk = 0;
static button = 0;

volatile uint8_t state = 1;
volatile uint16_t Battery_voltage,Battery_current,FSR_voltage;

#define TIMER_INTERVAL_ACCEL_UPDATE     APP_TIMER_TICKS(5000) // 2000 ms intervals
volatile bool start_accel_update_flag = false;
APP_TIMER_DEF(m_timer_accel_update_id);

#define TIMER_INTERVAL_ADC_UPDATE     APP_TIMER_TICKS(3000) // 3000 ms intervals
volatile bool start_adc_update_flag = false;
APP_TIMER_DEF(m_timer_adc_update_id);

#define TIMER_INTERVAL_PWM_UPDATE     APP_TIMER_TICKS(500) // 500 ms intervals
APP_TIMER_DEF(m_timer_pwm_update_id);
volatile bool start_pwm_update_flag = false;


volatile bool bat_low = 0,charge=0,vibrate_on = 0;
volatile uint16_t Vibrate_value , Temp_value ,Vibrate_value2;

#ifdef NRF52810_XXAA
static const nrf_drv_timer_t   m_timer = NRF_DRV_TIMER_INSTANCE(2);
#else
static const nrf_drv_timer_t   m_timer = NRF_DRV_TIMER_INSTANCE(3);
#endif
static nrf_saadc_value_t       m_buffer_pool[2][SAADC_SAMPLES_IN_BUFFER];
static nrf_ppi_channel_t       m_ppi_channel;
static uint32_t                m_adc_evt_counter;

volatile uint32_t vibrate_count;
volatile uint32_t right;
volatile uint32_t left;

static bool bl_command=0;
static uint8_t bluetooth_command[4];

static uint8_t accel_values_array[6];
/* TWI instance ID. */
#define TWI_INSTANCE_ID     0

/* Common addresses definition for sensor. */
#define ICM20948_ADDR          0x68 //(0x90U >> 1)
#define MPU_REG_ACCEL_XOUT_H   0x2D

/* Indicates if operation on TWI has ended. */
static volatile bool m_xfer_done = false;

/* TWI instance. */
static const nrf_drv_twi_t m_twi = NRF_DRV_TWI_INSTANCE(TWI_INSTANCE_ID);


extern void Convert_to_ascii(uint16_t num,uint8_t *num_array,uint8_t n);
extern void  Convert_Ascii_array_to_int(uint8_t *ascii_array, uint8_t c);

/* Buffer for samples read from sensor. */
static uint8_t m_sample;

APP_PWM_INSTANCE(PWM1,1);                   // Create the instance "PWM1" using TIMER1.

static volatile bool ready_flag;            // A flag indicating PWM status.

void pwm_ready_callback(uint32_t pwm_id)    // PWM callback function
{
    ready_flag = true;
}


/**@brief Function for assert macro callback.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyse
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}

void timer_accel_update_handler(void * p_context)
{
    start_accel_update_flag = true;
}


void timer_adc_update_handler(void * p_context)
{
    start_adc_update_flag = true;
}

void timer_pwm_update_handler(void * p_context)
{
    start_pwm_update_flag = true;
}


/**@brief Function for initializing the timer module.
 */
static void timers_init(void)
{
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);
    err_code = app_timer_create(&m_timer_accel_update_id, APP_TIMER_MODE_REPEATED, timer_accel_update_handler);
    APP_ERROR_CHECK(err_code);

     err_code = app_timer_create(&m_timer_adc_update_id, APP_TIMER_MODE_REPEATED, timer_adc_update_handler);
    APP_ERROR_CHECK(err_code);

    err_code = app_timer_create(&m_timer_pwm_update_id, APP_TIMER_MODE_REPEATED, timer_pwm_update_handler);
    APP_ERROR_CHECK(err_code);



}

/**@brief Function for the GAP initialization.
 *
 * @details This function will set up all the necessary GAP (Generic Access Profile) parameters of
 *          the device. It also sets the permissions and appearance.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *) DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling Queued Write Module errors.
 *
 * @details A pointer to this function will be passed to each service which may need to inform the
 *          application about an error.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void nrf_qwr_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for handling the data from the Nordic UART Service.
 *
 * @details This function will process the data received from the Nordic UART BLE Service and send
 *          it to the UART module.
 *
 * @param[in] p_evt       Nordic UART Service event.
 */
/**@snippet [Handling the data received over BLE] */

static volatile bool send_time_update = false;

static void nus_data_handler(ble_nus_evt_t * p_evt)
{
NRF_LOG_INFO("BLINKY START\n");
      uint8_t vib_cnt_buf[5],vibrate_count_array[3],vvv[3];
      ret_code_t err_code;
      uint16_t length = 8,i=0,len1=2,len2=6,ha = 3,lendt = 16;
      uint8_t message1[]= "OK" ;
      uint8_t message2[]= "NOT OK" ;


    if (p_evt->type == BLE_NUS_EVT_RX_DATA)
    {
        uint32_t err_code;


      if (p_evt->params.rx_data.p_data[0]=='O')
      {   button = 1;
      }


if(p_evt->params.rx_data.p_data[0] == 'T')
        {
            send_time_update = true;
        }


//----------------------------higher and lower threshold--------------------------------------

    
      if (p_evt->params.rx_data.p_data[0]=='L')
      {
          memcpy(&vvv[0], (&p_evt->params.rx_data.p_data[1]),3);
             Convert_Ascii_array_to_int(&vvv[0],3);
             Vibrate_value2 = Temp_value;

      }

      if (p_evt->params.rx_data.p_data[0]=='H')
      {
          memcpy(&vvv[0], (&p_evt->params.rx_data.p_data[1]),3);
             Convert_Ascii_array_to_int(&vvv[0],3);
             Vibrate_value = Temp_value;
             Convert_to_ascii(Temp_value,(&vib_cnt_buf[0]),3);
             

      }
//-------------------------------------------------------------------------------------------------

//-------------------------leg??-----------------------------------------------
    if ((p_evt->params.rx_data.p_data[0]=='l')&&(p_evt->params.rx_data.p_data[1]=='e')&&(p_evt->params.rx_data.p_data[2]=='g'))
    {
      if(p_evt->params.rx_data.p_data[3]=='L')
        kk = 1;
      else
        kk = 0;
    }

//---------------------------------------------
      
      
       if(p_evt->params.rx_data.p_data[0]=='P')
       {
          memcpy(&vvv[0], (&p_evt->params.rx_data.p_data[1]),3);
          Convert_Ascii_array_to_int(&vvv[0],3);
          dc = Temp_value;
       }
      
      
       if((p_evt->params.rx_data.p_data[0]=='a')&&(p_evt->params.rx_data.p_data[1]=='a')) 
                   {
              bl_command = 1;
              
              if (kk == 0)
            {
              Convert_to_ascii(right,(&vib_cnt_buf[0]),8);
              Convert_to_ascii(left,(&vib_cnt_buf[2]),2);
              //err_code = ble_nus_data_send(&m_nus, (&vib_cnt_buf[0]), &length, m_conn_handle);
              err_code = ble_nus_data_send(&m_nus, (&vib_cnt_buf), &length, m_conn_handle);
            }
           
            else

            {
              Convert_to_ascii(right,(&vib_cnt_buf[0]),8);
              Convert_to_ascii(left,(&vib_cnt_buf[2]),2);
              //err_code = ble_nus_data_send(&m_nus, (&vib_cnt_buf[0]), &length, m_conn_handle);
              err_code = ble_nus_data_send(&m_nus, (&vib_cnt_buf), &length, m_conn_handle);
            }


          }
           else if((p_evt->params.rx_data.p_data[0]=='*')&&(p_evt->params.rx_data.p_data[1]=='*'))
          {
           //  for(i=0;i<3;i++)
           //   vibrate_count_array[i] = p_evt->params.rx_data.p_data[1+i];
             memcpy(&vibrate_count_array[0], (&p_evt->params.rx_data.p_data[2]),3);
             Convert_Ascii_array_to_int(&vibrate_count_array[0],3);
            if((Temp_value > 150 ) & (Temp_value < 999 ))
            {   
               Vibrate_value = Temp_value ;

              err_code = ble_nus_data_send(&m_nus, message1, &len1, m_conn_handle);
              }
              else
              {

              err_code = ble_nus_data_send(&m_nus, message2, &len2, m_conn_handle);
              }

              
          }
          else if((p_evt->params.rx_data.p_data[0]=='&')&&(p_evt->params.rx_data.p_data[1]=='&'))
          {
           //  for(i=0;i<3;i++)
           //   vibrate_count_array[i] = p_evt->params.rx_data.p_data[1+i];
             memcpy(&vibrate_count_array[0], (&p_evt->params.rx_data.p_data[2]),3);
             Convert_Ascii_array_to_int(&vibrate_count_array[0],3);
            if((Temp_value > 50 ) & (Temp_value < 150 ))
            {   
               Vibrate_value2 = Temp_value ;

              err_code = ble_nus_data_send(&m_nus, message1, &len1, m_conn_handle);
              }
              else
              {

              err_code = ble_nus_data_send(&m_nus, message2, &len2, m_conn_handle);
              }

              
          }
            
    }


    
       
}
/**@snippet [Handling the data received over BLE] */


/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    uint32_t           err_code;
    ble_nus_init_t     nus_init;
    nrf_ble_qwr_init_t qwr_init = {0};

    // Initialize Queued Write Module.
    qwr_init.error_handler = nrf_qwr_error_handler;

    err_code = nrf_ble_qwr_init(&m_qwr, &qwr_init);
    APP_ERROR_CHECK(err_code);

    // Initialize NUS.
    memset(&nus_init, 0, sizeof(nus_init));

    nus_init.data_handler = nus_data_handler;

    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling an event from the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module
 *          which are passed to the application.
 *
 * @note All this function does is to disconnect. This could have been done by simply setting
 *       the disconnect_on_fail config parameter, but instead we use the event handler
 *       mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */


static void peer_manager_init(void)
{
    ble_gap_sec_params_t sec_param;
    ret_code_t           err_code;

    err_code = pm_init();
    APP_ERROR_CHECK(err_code);

    memset(&sec_param, 0, sizeof(ble_gap_sec_params_t));

    // Security parameters to be used for all security procedures.
    sec_param.bond           = SEC_PARAM_BOND;
    sec_param.mitm           = SEC_PARAM_MITM;
    sec_param.lesc           = SEC_PARAM_LESC;
    sec_param.keypress       = SEC_PARAM_KEYPRESS;
    sec_param.io_caps        = SEC_PARAM_IO_CAPABILITIES;
    sec_param.oob            = SEC_PARAM_OOB;
    sec_param.min_key_size   = SEC_PARAM_MIN_KEY_SIZE;
    sec_param.max_key_size   = SEC_PARAM_MAX_KEY_SIZE;
    sec_param.kdist_own.enc  = 1;
    sec_param.kdist_own.id   = 1;
    sec_param.kdist_peer.enc = 1;
    sec_param.kdist_peer.id  = 1;

    err_code = pm_sec_params_set(&sec_param);
    APP_ERROR_CHECK(err_code);

    err_code = pm_register(pm_evt_handler);
    APP_ERROR_CHECK(err_code);
}


/**@brief Clear bond information from persistent storage.
 */
static void delete_bonds(void)
{
    ret_code_t err_code;

    NRF_LOG_INFO("Erase bonds!");

    err_code = pm_peers_delete();
    APP_ERROR_CHECK(err_code);
}




static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
    }
}


/**@brief Function for handling errors from the Connection Parameters module.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for putting the chip into sleep mode.
 *
 * @note This function will not return.
 */
static void sleep_mode_enter(void)
{
    uint32_t err_code = bsp_indication_set(BSP_INDICATE_IDLE);
    APP_ERROR_CHECK(err_code);

    // Prepare wakeup buttons.
    err_code = bsp_btn_ble_sleep_mode_prepare();
    APP_ERROR_CHECK(err_code);

    // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_power_system_off();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
    uint32_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
            err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
            APP_ERROR_CHECK(err_code);
            break;
        case BLE_ADV_EVT_IDLE:
           // sleep_mode_enter();
            break;
        default:
            break;
    }
}


/**@brief Function for handling BLE events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 * @param[in]   p_context   Unused.
 */
static void ble_evt_handler(ble_evt_t const * p_ble_evt, void * p_context)
{
    uint32_t err_code;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            NRF_LOG_INFO("Connected");
            //err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            //APP_ERROR_CHECK(err_code);
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            err_code = nrf_ble_qwr_conn_handle_assign(&m_qwr, m_conn_handle);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            NRF_LOG_INFO("Disconnected");
            // LED indication will be changed when advertising starts.
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            break;

        case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
        {
            NRF_LOG_DEBUG("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
            APP_ERROR_CHECK(err_code);
        } break;

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            // Pairing not supported
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for the SoftDevice initialization.
 *
 * @details This function initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    ret_code_t err_code;

    err_code = nrf_sdh_enable_request();
    APP_ERROR_CHECK(err_code);

    // Configure the BLE stack using the default settings.
    // Fetch the start address of the application RAM.
    uint32_t ram_start = 0;
    err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
    APP_ERROR_CHECK(err_code);

    // Enable BLE stack.
    err_code = nrf_sdh_ble_enable(&ram_start);
    APP_ERROR_CHECK(err_code);

    // Register a handler for BLE events.
    NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
}


/**@brief Function for handling events from the GATT library. */
void gatt_evt_handler(nrf_ble_gatt_t * p_gatt, nrf_ble_gatt_evt_t const * p_evt)
{
    if ((m_conn_handle == p_evt->conn_handle) && (p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED))
    {
        m_ble_nus_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
        NRF_LOG_INFO("Data len is set to 0x%X(%d)", m_ble_nus_max_data_len, m_ble_nus_max_data_len);
    }
    NRF_LOG_DEBUG("ATT MTU exchange completed. central 0x%x peripheral 0x%x",
                  p_gatt->att_mtu_desired_central,
                  p_gatt->att_mtu_desired_periph);
}


/**@brief Function for initializing the GATT library. */
void gatt_init(void)
{
    ret_code_t err_code;

    err_code = nrf_ble_gatt_init(&m_gatt, gatt_evt_handler);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_gatt_att_mtu_periph_set(&m_gatt, NRF_SDH_BLE_GATT_MAX_MTU_SIZE);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated by button press.
 */
void bsp_event_handler(bsp_event_t event)
{
    uint32_t err_code;
    switch (event)
    {
        case BSP_EVENT_SLEEP:
            sleep_mode_enter();
            break;

        case BSP_EVENT_DISCONNECT:
            err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
            break;

        case BSP_EVENT_WHITELIST_OFF:
            if (m_conn_handle == BLE_CONN_HANDLE_INVALID)
            {
                err_code = ble_advertising_restart_without_whitelist(&m_advertising);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
            }
            break;

        default:
            break;
    }
}


/**@brief   Function for handling app_uart events.
 *
 * @details This function will receive a single character from the app_uart module and append it to
 *          a string. The string will be be sent over BLE when the last character received was a
 *          'new line' '\n' (hex 0x0A) or if the string has reached the maximum data length.
 */
/**@snippet [Handling the data received over UART] */
void uart_event_handle(app_uart_evt_t * p_event)
{
    static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
    static uint8_t index = 0;
    uint32_t       err_code;

    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY:
            UNUSED_VARIABLE(app_uart_get(&data_array[index]));
            index++;

            if ((data_array[index - 1] == '\n') ||
                (data_array[index - 1] == '\r') ||
                (index >= m_ble_nus_max_data_len))
            {
                if (index > 1)
                {
                    NRF_LOG_DEBUG("Ready to send data over BLE NUS");
                    NRF_LOG_HEXDUMP_DEBUG(data_array, index);

                    do
                    {
                        uint16_t length = (uint16_t)index;
                        err_code = ble_nus_data_send(&m_nus, data_array, &length, m_conn_handle);
                        if ((err_code != NRF_ERROR_INVALID_STATE) &&
                            (err_code != NRF_ERROR_RESOURCES) &&
                            (err_code != NRF_ERROR_NOT_FOUND))
                        {
                            APP_ERROR_CHECK(err_code);
                        }
                    } while (err_code == NRF_ERROR_RESOURCES);
                }

                index = 0;
            }
            break;

        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}
/**@snippet [Handling the data received over UART] */


/**@brief  Function for initializing the UART module.
 */
/**@snippet [UART Initialization] */
static void uart_init(void)
{
    uint32_t                     err_code;
    app_uart_comm_params_t const comm_params =
    {
        .rx_pin_no    = RX_PIN_NUMBER,
        .tx_pin_no    = TX_PIN_NUMBER,
        .rts_pin_no   = RTS_PIN_NUMBER,
        .cts_pin_no   = CTS_PIN_NUMBER,
        .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
        .use_parity   = false,
#if defined (UART_PRESENT)
        .baud_rate    = NRF_UART_BAUDRATE_115200
#else
        .baud_rate    = NRF_UARTE_BAUDRATE_115200
#endif
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOWEST,
                       err_code);
    APP_ERROR_CHECK(err_code);
}
/**@snippet [UART Initialization] */


/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init(void)
{
    uint32_t               err_code;
    ble_advertising_init_t init;

    memset(&init, 0, sizeof(init));

    init.advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    init.advdata.include_appearance = false;
    init.advdata.flags              = BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;

    init.srdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    init.srdata.uuids_complete.p_uuids  = m_adv_uuids;

    init.config.ble_adv_fast_enabled  = true;
    init.config.ble_adv_fast_interval = APP_ADV_INTERVAL;
    init.config.ble_adv_fast_timeout  = APP_ADV_DURATION;
    init.evt_handler = on_adv_evt;

    err_code = ble_advertising_init(&m_advertising, &init);
    APP_ERROR_CHECK(err_code);

   ble_advertising_conn_cfg_tag_set(&m_advertising, APP_BLE_CONN_CFG_TAG);
}


/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init(bool * p_erase_bonds)
{
    bsp_event_t startup_event;

    uint32_t err_code = bsp_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS, bsp_event_handler);
    APP_ERROR_CHECK(err_code);

    err_code = bsp_btn_ble_init(NULL, &startup_event);
    APP_ERROR_CHECK(err_code);

    *p_erase_bonds = (startup_event == BSP_EVENT_CLEAR_BONDING_DATA);
}


/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}


/**@brief Function for initializing power management.
 */
static void power_management_init(void)
{
    ret_code_t err_code;
    err_code = nrf_pwr_mgmt_init();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the idle state (main loop).
 *
 * @details If there is no pending log operation, then sleep until next the next event occurs.
 */
static void idle_state_handle(void)
{
    UNUSED_RETURN_VALUE(NRF_LOG_PROCESS());
    nrf_pwr_mgmt_run();
}


/**@brief Function for starting advertising.
 */
static void advertising_start(bool erase_bonds)
{
    if (erase_bonds == true)
    {
        delete_bonds();
        // Advertising is started by PM_EVT_PEERS_DELETE_SUCCEEDED event.
    }
    else
    {
        ret_code_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);

        APP_ERROR_CHECK(err_code);
    }
}


//////////////////////////////////////////////////////
uint32_t bluetooth_sleep(void)
{
	uint32_t err_code;

	// If connected, disconnect
	if (m_conn_handle != BLE_CONN_HANDLE_INVALID)
	{
		err_code = sd_ble_gap_disconnect(m_conn_handle,  BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
		if (err_code != NRF_SUCCESS) return err_code;
	}

	// Stop advertising
	err_code = sd_ble_gap_adv_stop(m_advertising.adv_handle);
	if (err_code != NRF_SUCCESS) return err_code;

	// Disable the radio tasks as scytulip suggested
	// devzone.nordicsemi.com/.../
	NRF_RADIO->TASKS_DISABLE;

	return NRF_SUCCESS;
}

/////////////////////////////////////////////////////////////////////

extern void Convert_to_ascii(uint16_t num,uint8_t *num_array,uint8_t n)
{

    uint8_t _i; 	
	_i = n;
     // _i = 0;
	do{
	_i-- ;			
	num_array[_i] = (num % 10)+'0';
       	num /= 10 ; 
       // _i++;
		
	}while(_i != 0);
   
}

extern void  Convert_Ascii_array_to_int(uint8_t *ascii_array, uint8_t c)
{
   uint16_t value=0;
   uint8_t i,hex;
   
   uint16_t mf = 1;
 i = c;
	 do{ 
	i--;	
	hex = ascii_array[i]-'0';
	value = value + hex*mf ; 
        mf = mf*10 ;
	}while(i != 0);
//return(value);  	
Temp_value = value ;
}


void timer_handler(nrf_timer_event_t event_type, void* p_context)
{

}


void saadc_sampling_event_init(void)
{
    ret_code_t err_code;
    err_code = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code);
    
    nrf_drv_timer_config_t timer_config = NRF_DRV_TIMER_DEFAULT_CONFIG;
    timer_config.frequency = NRF_TIMER_FREQ_31250Hz;
    err_code = nrf_drv_timer_init(&m_timer, &timer_config, timer_handler);
    APP_ERROR_CHECK(err_code);

    /* setup m_timer for compare event */
    uint32_t ticks = nrf_drv_timer_ms_to_ticks(&m_timer,SAADC_SAMPLE_RATE);
    nrf_drv_timer_extended_compare(&m_timer, NRF_TIMER_CC_CHANNEL0, ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, false);
    nrf_drv_timer_enable(&m_timer);

    uint32_t timer_compare_event_addr = nrf_drv_timer_compare_event_address_get(&m_timer, NRF_TIMER_CC_CHANNEL0);
    uint32_t saadc_sample_event_addr = nrf_drv_saadc_sample_task_get();

    /* setup ppi channel so that timer compare event is triggering sample task in SAADC */
    err_code = nrf_drv_ppi_channel_alloc(&m_ppi_channel);
    APP_ERROR_CHECK(err_code);
    
    err_code = nrf_drv_ppi_channel_assign(m_ppi_channel, timer_compare_event_addr, saadc_sample_event_addr);
    APP_ERROR_CHECK(err_code);
}


void saadc_sampling_event_enable(void)
{
    ret_code_t err_code = nrf_drv_ppi_channel_enable(m_ppi_channel);
    APP_ERROR_CHECK(err_code);
}


void saadc_callback(nrf_drv_saadc_evt_t const * p_event)
{
    if (p_event->type == NRF_DRV_SAADC_EVT_DONE)
    {
        ret_code_t err_code;
        uint16_t adc_value,mV;
        float  mV_value_digit;
        uint8_t value[SAADC_SAMPLES_IN_BUFFER*2];
        uint16_t bytes_to_send;
        uint8_t mV_values[20] = {"    ,    ,    ,     "};
        uint16_t send_bytes = 20;
     
        // set buffers
        err_code = nrf_drv_saadc_buffer_convert(p_event->data.done.p_buffer, SAADC_SAMPLES_IN_BUFFER);
        APP_ERROR_CHECK(err_code);
						
        // print samples on hardware UART and parse data for BLE transmission
       // printf("ADC event number: %d\r\n",(int)m_adc_evt_counter);
        for (int i = 0; i < SAADC_SAMPLES_IN_BUFFER; i++)
        {
           // printf("%d\r\n", p_event->data.done.p_buffer[i]);

            adc_value = p_event->data.done.p_buffer[i];
            if(adc_value > 4096)
                adc_value = 0;
            mV_value_digit = adc_value ;
            switch(i)
              {
                case 0:              
                      mV_value_digit = mV_value_digit * 1.75 ;
                      Battery_voltage = mV_value_digit;
                      break;
               case 1:
                      mV_value_digit = mV_value_digit * 0.88 ;
                      Battery_current = mV_value_digit;
                      break;
               case 3:
                      mV_value_digit = mV_value_digit * 0.88 ;
                      FSR_voltage = mV_value_digit;
                      break;
                    
              }
              
           // mV_value_digit = mV_value_digit / 4096 ;
           // mV_value_digit = mV_value_digit/ 1000 ;
             mV = mV_value_digit ;
            Convert_to_ascii(mV,(&mV_values[i*5]),4);

            value[i*2] = adc_value;
            value[(i*2)+1] = adc_value >> 8;
        }
				
        // Send data over BLE via NUS service. Makes sure not to send more than 20 bytes.
        if((SAADC_SAMPLES_IN_BUFFER*2) <= 20) 
        {
            bytes_to_send = (SAADC_SAMPLES_IN_BUFFER*2);
        }
        else 
        {
            bytes_to_send = 20;
        }
       // err_code = ble_nus_data_send(&m_nus, value, &bytes_to_send, m_conn_handle);
     /* err_code = ble_nus_data_send(&m_nus, (&mV_values[0]), &send_bytes, m_conn_handle);
      if ((err_code != NRF_ERROR_INVALID_STATE) && (err_code != NRF_ERROR_NOT_FOUND)) 
        {
            APP_ERROR_CHECK(err_code);
        }*/
						
        m_adc_evt_counter++;
    }
}


void saadc_init(void)
{
    ret_code_t err_code;
	
    nrf_drv_saadc_config_t saadc_config = NRF_DRV_SAADC_DEFAULT_CONFIG;
    saadc_config.resolution = NRF_SAADC_RESOLUTION_12BIT;
	
    nrf_saadc_channel_config_t channel_0_config =
        NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN2);
    channel_0_config.gain = NRF_SAADC_GAIN1_6;
    channel_0_config.reference = NRF_SAADC_REFERENCE_INTERNAL;
	
    nrf_saadc_channel_config_t channel_1_config =
        NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN3);
    channel_1_config.gain = NRF_SAADC_GAIN1_6;
    channel_1_config.reference = NRF_SAADC_REFERENCE_INTERNAL;
	
    nrf_saadc_channel_config_t channel_2_config =
     NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN4);
    channel_2_config.gain = NRF_SAADC_GAIN1_6;
    channel_2_config.reference = NRF_SAADC_REFERENCE_INTERNAL;
	
    nrf_saadc_channel_config_t channel_3_config =
        NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN5);
    channel_3_config.gain = NRF_SAADC_GAIN1_6;
    channel_3_config.reference = NRF_SAADC_REFERENCE_INTERNAL;				
	
    err_code = nrf_drv_saadc_init(&saadc_config, saadc_callback);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_saadc_channel_init(0, &channel_0_config);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_saadc_channel_init(1, &channel_1_config);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_saadc_channel_init(2, &channel_2_config);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_saadc_channel_init(3, &channel_3_config);
    APP_ERROR_CHECK(err_code);	

    err_code = nrf_drv_saadc_buffer_convert(m_buffer_pool[0],SAADC_SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(err_code);   
    err_code = nrf_drv_saadc_buffer_convert(m_buffer_pool[1],SAADC_SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(err_code);
}
/**
 * @brief Function for handling data from temperature sensor.
 *
 * @param[in] temp          Temperature in Celsius degrees read from sensor.
 */
__STATIC_INLINE void data_handler(uint8_t temp)
{
    //NRF_LOG_INFO("Temperature: %d Celsius degrees.", temp);
}

/**
 * @brief TWI events handler.
 */
void twi_handler(nrf_drv_twi_evt_t const * p_event, void * p_context)
{
    switch (p_event->type)
    {
        case NRF_DRV_TWI_EVT_DONE:
            if (p_event->xfer_desc.type == NRF_DRV_TWI_XFER_RX)
            {
                data_handler(m_sample);
            }
            m_xfer_done = true;
            break;
        default:
            break;
    }
}

/**
 * @brief twi initialization.
 */
void twi_init (void)
{
    ret_code_t err_code;

    const nrf_drv_twi_config_t twi_ICM20948_config = {
       .scl                = ARDUINO_SCL_PIN,
       .sda                = ARDUINO_SDA_PIN,
       .frequency          = NRF_DRV_TWI_FREQ_100K,
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH,
       .clear_bus_init     = false
    };

    err_code = nrf_drv_twi_init(&m_twi, &twi_ICM20948_config, twi_handler, NULL);
    APP_ERROR_CHECK(err_code);

    nrf_drv_twi_enable(&m_twi);
}



static void application_timers_start(void)
{
  uint32_t err_code;
    //uint32_t err_code = app_timer_start(m_timer_accel_update_id, TIMER_INTERVAL_ACCEL_UPDATE, NULL);
    //APP_ERROR_CHECK(err_code);

    err_code = app_timer_start(m_timer_adc_update_id, TIMER_INTERVAL_ACCEL_UPDATE, NULL);
    APP_ERROR_CHECK(err_code);

    


}

void Port_init(void)
{
          nrf_gpio_cfg_output(BRD_LED1);
	  nrf_gpio_cfg_output(BRD_LED2);
          nrf_gpio_cfg_output(BRD_LED3);
	 // nrf_gpio_cfg_output(BRD_LED4);
        //  nrf_gpio_cfg_output(MOT_CTL); 
	
	/*  nrf_gpio_pin_set(BRD_LED1);
          nrf_gpio_pin_set(BRD_LED2);
          nrf_gpio_pin_set(BRD_LED3);
          nrf_gpio_pin_set(BRD_LED4);*/
        //  nrf_gpio_pin_clear(MOT_CTL); 

          nrf_gpio_cfg_input(BAT_STATE, BUTTON_PULL);
          nrf_gpio_cfg_input(KEY_PIN, BUTTON_PULL);

}

void ICM20948_set_mode(void)
{
    ret_code_t err_code;

    /* Writing to LM75B_REG_CONF "0" set temperature sensor in NORMAL mode. */
    uint8_t reg[2] = {0x6, 00};
    err_code = nrf_drv_twi_tx(&m_twi, ICM20948_ADDR, reg, sizeof(reg), false);
    APP_ERROR_CHECK(err_code);
    while (m_xfer_done == false);

    /* Writing to pointer byte. */
  //  reg[0] = LM75B_REG_TEMP;
  //  m_xfer_done = false;
  //  err_code = nrf_drv_twi_tx(&m_twi, MPU_ADDR, reg, 1, false);
  //  APP_ERROR_CHECK(err_code);
  //  while (m_xfer_done == false);
}

uint32_t app_mpu_read_accel(accel_values_t * accel_values)
{
    uint32_t err_code;
    uint8_t raw_values[6];
    uint8_t *data;
    uint8_t reg = 0x2D;

   // err_code = nrf_drv_mpu_read_registers(MPU_REG_ACCEL_XOUT_H, raw_values, 6);
   // if(err_code != NRF_SUCCESS) return err_code;


    err_code = nrf_drv_twi_tx(&m_twi, ICM20948_ADDR, &reg, 1, false);
   // APP_ERROR_CHECK(err_code);
    while (m_xfer_done == false);

     m_xfer_done = false;
     nrf_delay_ms(100);

     err_code = nrf_drv_twi_rx(&m_twi, ICM20948_ADDR,( &raw_values[0]), 7);
   
     memcpy(accel_values_array,raw_values,6);
    
   /* data = (uint8_t*)accel_values;
    for(uint8_t i = 0; i<6; i++)
    {
        *data = raw_values[5-i];
         accel_values_array[i] =  raw_values[i] ;
        data++;
    }*/
    return NRF_SUCCESS;
}

void pwm_init(void)
{
  ret_code_t err_code;

  app_pwm_config_t pwm1_cfg = APP_PWM_DEFAULT_CONFIG_1CH(1000L,MOT_CTL );
  pwm1_cfg.pin_polarity[0] = APP_PWM_POLARITY_ACTIVE_HIGH;
   /* Initialize and enable PWM. */
    err_code = app_pwm_init(&PWM1,&pwm1_cfg,pwm_ready_callback);
    APP_ERROR_CHECK(err_code);
    app_pwm_enable(&PWM1);

}

/**@brief Application main function.
 */

 


int main(void)
{
    APP_ERROR_CHECK(NRF_LOG_INIT(NULL));
    NRF_LOG_DEFAULT_BACKENDS_INIT();

    //NRF_LOG_INFO("BSP example started.");
    //NRF_LOG_FLUSH();
    bool erase_bonds;
    bool pwm_on;
     uint16_t send_bytes= 4;
     uint8_t message[] = {"Hello"};
     uint8_t vib_cnt_buf[5];
     ret_code_t err_code;
      accel_values_t accel_values;
      Vibrate_value = 800;
      Vibrate_value2 = 80;
    

    // Initialize.
   // uart_init();
   // log_init();
    timers_init();
    buttons_leds_init(&erase_bonds);
    Port_init();
    power_management_init();
    ble_stack_init();
    gap_params_init();
    gatt_init();
    services_init();
    advertising_init();
    conn_params_init();
    peer_manager_init();

    saadc_sampling_event_init();
    saadc_init();
    saadc_sampling_event_enable();

    twi_init();
    nrf_gpio_pin_clear(BRD_LED3);
    nrf_gpio_pin_clear(BRD_LED1);
    pwm_init();
    application_timers_start();  
      advertising_start(erase_bonds);
 
    //  ICM20948_set_mode();
    
nrf_cal_set_time(2020, 7, 20, 12, 30, 0);

nrf_delay_ms(1000);


    // Enter main loop.
    for (;;)
    {

     if(button == 1) {
    uint32_t h = bluetooth_sleep();}

//////////////////////////

if(send_time_update)
        {
            char *time_string;
            uint16_t length;
            send_time_update = false;
            
            time_string = nrf_cal_get_time_string(true);
            length = strlen(time_string);
            uint32_t err_code = ble_nus_data_send(&m_nus, time_string, &length, m_conn_handle);
            APP_ERROR_CHECK(err_code);
        }

//////////////



if((FSR_voltage >= Vibrate_value) | (FSR_voltage < Vibrate_value2))
{
	if(vibrate_on  != 1)
	{	
            vibrate_on = 1;
            nrf_delay_ms(1000);
           err_code = app_timer_start(m_timer_pwm_update_id, TIMER_INTERVAL_PWM_UPDATE, NULL);
           //APP_ERROR_CHECK(err_code);
           pwm_on = 1;
           
           app_pwm_enable(&PWM1);
            while (app_pwm_channel_duty_set(&PWM1, 0, dc) == NRF_ERROR_BUSY);
          if(FSR_voltage >= Vibrate_value)
           {

          

           if (kk==0)
            right++;
           else
           left++;
            }
            else if(FSR_voltage < Vibrate_value2)
            {
              if (kk==0)
              left++;
              else
              right++;
            }
          
         }
}
else
{
   
	if(vibrate_on  == 1)
		{
                  vibrate_on  = 0 ;
                 app_pwm_disable(&PWM1);
                  err_code = app_timer_stop(m_timer_pwm_update_id);
                 }
}





if(vibrate_on  == 1)
{
if(start_pwm_update_flag == 1)
{
  if(pwm_on == 1)
  {
     pwm_on = 0;
     app_pwm_disable(&PWM1);

  }
  else
  {
   pwm_on = 1;
            

 app_pwm_enable(&PWM1);
   while (app_pwm_channel_duty_set(&PWM1, 0, dc) == NRF_ERROR_BUSY);

  }
  start_pwm_update_flag = 0;
  }
}


  if(start_adc_update_flag == true){

  if(nrf_gpio_pin_read(BAT_STATE)){
            nrf_gpio_pin_clear(BRD_LED3);
            charge = 0;

      if(Battery_voltage < 3300)
      {
      if(bat_low != 1)
        {
          bat_low = 1 ; 
          nrf_gpio_pin_set(BRD_LED1);
          }
        
      }
      else{
      if(bat_low == 1)
        {
          bat_low = 0 ; 
         nrf_gpio_pin_clear(BRD_LED1);
      }
      }


        }
        else{
           if(charge != 1){
            charge = 1;
            nrf_gpio_pin_clear(BRD_LED1);
            nrf_gpio_pin_set(BRD_LED3);
            
            }

        }

  

  start_adc_update_flag = false;
  }

    }
}


/**
 * @}
 */

